% comparing two versions of Mackey-Glass simulations Sim1 and Sim2 
% w.r.t. to their divergence rate of trajectories (corresponding to 
% positive Lyapunov exponent. 
% The simulation Sim1 is the discretized simulation used in my Science
% submission.
% The simulation Sim2 is the simulation obtained by invoking Matlab's
% solver for delay differential equations (Matlab function dde23) with a
% fine stepsize. Sim2 is used here as a reference.



function futureLoopPoints = continuateSim1(seed, nmax, incrementsperUnit)
% this function continues an initial trajectory of the MG attractor ( =
% seed) for up to nmax time units. It returns the values of the MG
% trajectory for times 1, ..., nmax, in a column vector futureLoopPoints
% of length nmax. incrementsperUnit is the number of simulation
% steps for one time unit of simulated attractor evolution.  Seed gives a
% MG trajectory section up to time n = 0. The length of this trajectory
% section is 17 time units, the delay length used in the Science article.
% Thus, seed is a column vector of length 17 * incrementsperUnit + 1. Argument
% nmax is an integer. 

tau = 17;
genHistoryLength = tau * incrementsperUnit ;
oldval = seed(length(seed),1);
genHistory = seed(1:length(seed)-1,1);
future = zeros(nmax,1);
step = 0;
if length(seed) ~= genHistoryLength + 1
    disp('Warning: conflicting seed and incrementsperUnit');
end
for n = 1:nmax
    for i = 1:incrementsperUnit
        step = step + 1;
        tauval = genHistory(mod(step,genHistoryLength)+1,1);
        newval = oldval + (0.2 * tauval/(1.0 + tauval^10) - 0.1 * oldval)/incrementsperUnit;
        genHistory(mod(step,genHistoryLength)+1,1) = oldval;
        oldval = newval;
    end    
        future(n,1) = newval;    
end
futureLoopPoints = future;


