function wm = generateHomSpatialWM(l,r)
% generates a weight matrix wm of size l^2 by l^2, with 
% random local connectivity pattern of radius r.
% The idea is that the wm reflects a retinal structure of l^2 neurons.
% The l^2 neurons are ordered row-wise in the l by l retina, 
% from left to right within rows, first row is at bottom.
% The generated weight matrix has spectral radius 1
% The weights all have constant absolute size
wm = zeros(l*l,l*l);
for i = 1:l*l
    for j = 1:l*l
        d = dist(i,j,l);
        if d < r
            wm(i,j) = sign(rand  - .5);
        end
    end
end
maxval = max(abs(eig(wm)));
wm = wm/maxval;



function d = dist(i,j,l)
% i, j are indices of neurons of wm, ranging from 1 to l*l
[x1,y1] = cartcoord(i,l);
[x2,y2] = cartcoord(j,l);
d = sqrt((x1-x2)^2 + (y1-y2)^2);

function [x,y] = cartcoord(i,l)
x = mod(i,l)+1;
y = ceil(i/l);


