% generateNet; 
 intWM0 = intWM;
specrad = 0.85;
intWM = specrad * intWM0;

% training sequence trainseq must have been produced by earlier call of
% generateMGSequence
sampleout = trainseq';

noiselevel = 0.0000000001;
linearOutputUnits = 0; % 1 = use linear output units, 0 = sigmoid output units
linearNetwork = 0; % 1 = use liner units in DR, 0 = use tanh units

% 1 = compute linear regression directly by solving Wiener Hopf equation
% with inverting covariance matrix; 0 = compute linear regression via
% pseudoinverse. 1 is much faster and more space-economical, but less accurate. 
WienerHopf = 0; 
initialRunlength = 1000;
sampleRunlength = 2000;
freeRunlength = 0;
plotRunlength = floor(200);
sparsePlotting = 0; %1= true, 0=false
plotStates = [1 2 3 4]; % plot internal network states of ...
% units indicated in row vector; maximally 4 are plotted


teacherscaling = 0.8;
teachershift = [0];


disp(sprintf('SR = %g   tsc = %g   tsh = %g   netsize = %g  noise = %g   N = %g',...
    specrad, teacherscaling, teachershift, netDim, noiselevel, ...
         sampleRunlength));


totalstate =  zeros(totalDim,1);    
internalState = totalstate(1:netDim);
outWM = initialOutWM;
stateCollectMat = zeros(sampleRunlength, netDim + inputLength);
teachCollectMat = zeros(sampleRunlength, outputLength);
teacherPL = zeros(outputLength, plotRunlength);
netOutPL = zeros(outputLength, plotRunlength);
if inputLength > 0
    inputPL = zeros(inputLength, plotRunlength);
end
statePL = zeros(length(plotStates),plotRunlength);
plotindex = 0;
msetest = zeros(1,outputLength); 
msetrain = zeros(1,outputLength); 


for i = 1:initialRunlength + sampleRunlength + freeRunlength + plotRunlength 
    
    if inputLength > 0
        in = [0.02];  % in is column vector  
    else in = [];
    end
    teach = [teacherscaling * sampleout(1,i) + teachershift];    % teach is column vector     
    
    %write input into totalstate
    if inputLength > 0
        totalstate(netDim+1:netDim+inputLength) = in; 
    end
    %update totalstate except at input positions  
    if linearNetwork
        if noiselevel == 0 |  i > initialRunlength + sampleRunlength
            internalState = ([intWM, inWM, ofbWM]*totalstate);  
        else
            internalState = ([intWM, inWM, ofbWM]*totalstate + ...
                noiselevel * 2.0 * (rand(netDim,1)-0.5));
            %             internalState = ([intWM, inWM, ofbWM]*totalstate + ...
            %              noiselevel * 2.0 * (randn(netDim,1)));
        end
    else
        if noiselevel == 0 |  i > initialRunlength + sampleRunlength
            internalState = f([intWM, inWM, ofbWM]*totalstate);  
        else
            internalState = f([intWM, inWM, ofbWM]*totalstate + ...
                noiselevel * 2.0 * (rand(netDim,1)-0.5));
            %             internalState = f([intWM, inWM, ofbWM]*totalstate + ...
            %              noiselevel * 2.0 * (randn(netDim,1)));
        end
    end
    
    if linearOutputUnits
        netOut = outWM *[internalState;in];
    else
        netOut = f(outWM *[internalState;in]);
    end
    totalstate = [internalState;in;netOut];    
    
    %collect states and results for later use in learning procedure
    if (i > initialRunlength) & (i <= initialRunlength + sampleRunlength) 
        collectIndex = i - initialRunlength;
        stateCollectMat(collectIndex,:) = [internalState' in']; %fill a row
        if linearOutputUnits
            teachCollectMat(collectIndex,:) = teach';
        else
            teachCollectMat(collectIndex,:) = (fInverse(teach))'; %fill a row
        end
    end
    %force teacher output 
    if i <= initialRunlength + sampleRunlength
        totalstate(netDim+inputLength+1:netDim+inputLength+outputLength) = teach' ; 
    end
    %update msetest
    if i > initialRunlength + sampleRunlength + freeRunlength
        for j = 1:outputLength
            msetest(1,j) = msetest(1,j) + (teach(j,1)- netOut(j,1))^2;
        end
    end
    %compute new model
    if i == initialRunlength + sampleRunlength
        if WienerHopf
            covMat = stateCollectMat' * stateCollectMat / sampleRunlength;
            pVec = stateCollectMat' * teachCollectMat / sampleRunlength;
            outWM = (inv(covMat) * pVec)';
        else
            outWM = (pinv(stateCollectMat) * teachCollectMat)'; 
        end
        %compute mean square errors on the training data using the newly
        %computed weights
        for j = 1:outputLength
            if linearOutputUnits
                msetrain(1,j) = sum((teachCollectMat(:,j) - ...
                    (stateCollectMat * outWM(j,:)')).^2);
            else
                msetrain(1,j) = sum((f(teachCollectMat(:,j)) - ...
                    f(stateCollectMat * outWM(j,:)')).^2);
            end
            msetrain(1,j) = msetrain(1,j) / sampleRunlength;
        end
    end  
    
    
    
    
    %write plotting data into various plotfiles
    if i > initialRunlength + sampleRunlength + freeRunlength 
        plotindex = plotindex + 1;
        if inputLength > 0
            inputPL(:,plotindex) = in;
        end
        teacherPL(:,plotindex) = teach; 
        netOutPL(:,plotindex) = netOut;
        for j = 1:length(plotStates)
            statePL(j,plotindex) = totalstate(plotStates(j),1);
        end
    end
end
%end of the great do-loop

% print diagnostics 

msetestresult = msetest / plotRunlength;
teacherVariance = var(teacherPL');
disp(sprintf('train NMSE = %g   test NMSE = %g   avWeights = %g', ...
    msetrain ./ teacherVariance, msetestresult ./ teacherVariance, mean(abs(outWM))));

% % input plot
% if inputLength == 1 & ~sparsePlotting
%     figure(1);
%     plot(inputPL(1,:));
%     title('final effective inputs','FontSize',8);
% end
% if inputLength == 2 & ~sparsePlotting
%     figure(1);
%     subplot(2,1,1);
%     plot(inputPL(1,:));
%     title('final effective inputs','FontSize',8);
%     subplot(2,1,2);
%     plot(inputPL(2,:));
% end
% if inputLength == 3 & ~sparsePlotting
%     figure(1);
%     subplot(3,1,1);
%     plot(inputPL(1,:));
%     title('final effective inputs','FontSize',8);
%     subplot(3,1,2);
%     plot(inputPL(2,:));
%     subplot(3,1,3);
%     plot(inputPL(3,:));
% end

% plot first 4 (maximally) of internal states listed in plotStates
if length(plotStates) > 0 & ~sparsePlotting
    figure(2);
    subplot(2,2,1);
    plot(statePL(1,:));
    title('internal states','FontSize',8);
    for k = 2:length(plotStates)
        subplot(2,2,k);
        plot(statePL(k,:));
    end    
end

% plot weights
if outputLength == 1 & ~sparsePlotting
    figure(3);
    plot(outWM(1,:));
    title('output weights','FontSize',8);
end
if outputLength == 2 & ~sparsePlotting
    figure(3);
    subplot(2,1,1);
    plot(outWM(1,:));
    title('output weights','FontSize',8);
    subplot(2,1,2);
    plot(outWM(2,:));
end
if outputLength == 3 & ~sparsePlotting
    figure(3);
    subplot(3,1,1);
    plot(outWM(1,:));
    title('output weights','FontSize',8);
    subplot(3,1,2);
    plot(outWM(2,:));
    subplot(3,1,3);
    plot(outWM(3,:));
end

% plot overlay of network output and teacher  
figure(4);
if outputLength == 1
    plot(1:plotRunlength,teacherPL(1,:),'-  k',...
        1:plotRunlength,netOutPL(1,:),': k');
    title('teacher vs. output','FontSize',8);
end
if outputLength == 2
    subplot(2,1,1);
    plot(1:plotRunlength,teacherPL(1,:),'-  k',...
        1:plotRunlength,netOutPL(1,:),': k');
    title('teacher vs. output','FontSize',8);
    subplot(2,1,2);
    plot(1:plotRunlength,teacherPL(2,:),'-  k',...
        1:plotRunlength,netOutPL(2,:),': k');
end
if outputLength == 3
    subplot(3,1,1);
    plot(1:plotRunlength,teacherPL(1,:),'-  k',...
        1:plotRunlength,netOutPL(1,:),': k');
    title('teacher vs. output','FontSize',8);
    subplot(3,1,2);
    plot(1:plotRunlength,teacherPL(2,:),'-  k',...
        1:plotRunlength,netOutPL(2,:),': k');
    subplot(3,1,3);
    plot(1:plotRunlength,teacherPL(3,:),'-  k',...
        1:plotRunlength,netOutPL(3,:),': k');
end








