% this script tests a learnt model of the MG tau17 attractor on numberOfTrials 
% portions of a test sequence testseq. Each of the portions is predicted
% for 84 time units. Several global vars need to have been determined
% before this script can be run by a call to learn.

sampleout = testseq';

% use the first 2000 step subsequence of testseq to prime model and 
% produce the network state after that
initialRunlength = 2000;

totalstate =  zeros(totalDim,1);    
internalState = totalstate(1:netDim);
teacherList = zeros( plotRunlength,1);
netOutList = zeros(plotRunlength,1);
plotindex = 0;

for i = 1:initialRunlength   
    in = 0.02;
    teach = [teacherscaling * sampleout(1,i) + teachershift];         
    %write input into totalstate
    totalstate(netDim+1:netDim+inputLength) = in; 
    %update totalstate except at input positions  
    internalState = f([intWM, inWM, ofbWM]*totalstate); 
    netOut = f(outWM *[internalState;in]);
    totalstate = [internalState;in;netOut];      
    totalstate(netDim+inputLength+1:netDim+inputLength+outputLength) = teach' ; 
end
startstate = totalstate;
% end of initial priming

testRunlength = 84;
numberOfTrials = 50;
outputCollectMat = zeros(testRunlength, numberOfTrials);
teacherCollectMat = zeros(testRunlength, numberOfTrials);
trialshift = 84;
for trials = 1:numberOfTrials
    % continue forced simulation run for trialshift steps
    totalstate = startstate;
    for i = 1:trialshift   
        in = 0.02;     
        teach = [teacherscaling * ...
                sampleout(1,initialRunlength + i + (trials - 1)*trialshift) + teachershift];         
        %write input into totalstate
        totalstate(netDim+1:netDim+inputLength) = in; 
        %update totalstate except at input positions  
        internalState = f([intWM, inWM, ofbWM]*totalstate); 
        netOut = f(outWM *[internalState;in]);
        totalstate = [internalState;in;netOut];      
        totalstate(netDim+inputLength+1:netDim+inputLength+outputLength) = teach' ; 
    end
    startstate = totalstate;
    
    for i = 1:testRunlength         
        in = 0.02;     
        teach = [teacherscaling * ...
                sampleout(1,initialRunlength + i + (trials - 1)*trialshift + trialshift) ...
                + teachershift];         
        %write input into totalstate
        totalstate(netDim+1:netDim+inputLength) = in; 
        %update totalstate except at input positions  
        internalState = f([intWM, inWM, ofbWM]*totalstate); 
        netOut = f(outWM *[internalState;in]);
        totalstate = [internalState;in;netOut];      
        outputCollectMat(i, trials) = netOut;
        teacherCollectMat(i, trials) = teach;
    end
end

% compute the normalized root MSE for 84 step prediction

% compute variance of original MG series
teachVar = var(fInverse(sampleout));
% undo the tanh transformation of the training data to recover (shifted)
% original MG data
outputCollectMatMG = fInverse(outputCollectMat);
teacherCollectMatMG = fInverse(teacherCollectMat);
% compute average NMSE of 84 step predictions
errors84 = outputCollectMatMG(84,:) - teacherCollectMatMG(84,:);
NRMSE84 = sqrt(mean(errors84.^2)/teachVar) % result 3.0999e-005

% 
% % compare divergence of model predictions over teacher with the theoretical
% % optimum, that is, the Lyapunov divergence
% initialPrecision = mean(abs(errors84))/exp(0.0059 * 84) % result 3.2173e-006
% % compute the number of steps at which we would have a divergence of 0.1
% % with an optimal (true) model, under numerical precision 10E-15
% stepsToDiv0Point1Opt = log(0.1/10^-15)/0.0059 % result 5.4638e+003


