% this little demo explains how one can weight "important"
% datapoints more than others, in the stateCollectMat and
% teachCollectMat matrices.

% Suppose you have collected states and teacher values into
% stateCollectMat and teachCollectMat, as follows:
% 
% stateCollectMat:
%      2     3     4     5
%      1     2     3     4
%      4     2     2     2
%      1     1     1     1
%      0     2     6     1
%      3     3     3     1
% teachCollectMat:
%      2
%      1
%      3
%      2
%      1
%      3
% 
% Now suppose further that you think the first data point is
% very important and you want to give it greater impact on 
% the final model. Let's say it is three times more important
% than the other data points. ONe way to incorporate this 
% importance into the calculation would be just to repeat the
% first datapoint entry three times, that is, use

% stateCollectMat:
%      2     3     4     5
%      2     3     4     5
%      2     3     4     5
%      1     2     3     4
%      4     2     2     2
%      1     1     1     1
%      0     2     6     1
%      3     3     3     1
% teachCollectMat:
%      2
%      2
%      2
%      1
%      3
%      2
%      1
%      3
     
% This would give you output weights as computed here:     
% 
% 

stateCollectMat = [ 2 3 4 5; 2 3 4 5; 2 3 4 5; 2 3 4 5; 1 2 3 4; 4 2 2 2; 1 1 1 1; 0 2 6 1; 3 3 3 1];
teachCollectMat = [ 5;5;5;5; 1; 3; 2; 1; 3];
outWM = (pinv(stateCollectMat) * teachCollectMat)'; 

% A nicer way to achieve exactly the same effect without duplicating 
% entries in stateCollectMat and teachCollectMat is to weigh the
% concerned entries by a factor of sqrt[n], where n is the 
% "importance" factor (n = 3 in the above example). This would give
% you 

w = sqrt(4);
stateCollectMat1 = [ w*2 w*3 w*4 w*5; 1 2 3 4; 4 2 2 2; 1 1 1 1; 0 2 6 1; 3 3 3 1];
teachCollectMat1 = [ w*5; 1; 3; 2; 1; 3];
outWM1 = (pinv(stateCollectMat1) * teachCollectMat1)'; 

% The results of both computations are identical, as is revealed
% in 

outWM - outWM1

